<?xml version="1.0"?>
<xsl:stylesheet version="1.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:xqx="http://www.w3.org/2005/04/XQueryX">

  <xsl:output method="text"/>
  <xsl:strip-space elements="*"/>

  <xsl:variable name="DOT">.</xsl:variable>
  <xsl:variable name="SPACE"><xsl:text> </xsl:text></xsl:variable>
  <xsl:variable name="SLASH"><xsl:text>/</xsl:text></xsl:variable>
  <xsl:variable name="SLASH_SLASH"><xsl:text>//</xsl:text></xsl:variable>
  <xsl:variable name="LESSTHAN"><xsl:text>&lt;</xsl:text></xsl:variable>
  <xsl:variable name="GREATERTHAN"><xsl:text>&gt;</xsl:text></xsl:variable>
  <xsl:variable name="LPAREN">(</xsl:variable>
  <xsl:variable name="RPAREN">)</xsl:variable>
  <xsl:variable name="NEWLINE"><xsl:text>
</xsl:text></xsl:variable>
  <xsl:variable name="COMMA">,</xsl:variable>
  <xsl:variable name="COMMA_SPACE">, </xsl:variable>
  <xsl:variable name="COMMA_NEWLINE"><xsl:text>,
</xsl:text></xsl:variable>
  <xsl:variable name="QUOTE"><xsl:text>'</xsl:text></xsl:variable>
  <xsl:variable name="DOUBLEQUOTE"><xsl:text>"</xsl:text></xsl:variable>
  <xsl:variable name="TRUE"><xsl:text>true</xsl:text></xsl:variable>
  <xsl:variable name="TO"><xsl:text> to </xsl:text></xsl:variable>
  <xsl:variable name="LBRACE"><xsl:text>{</xsl:text></xsl:variable>
  <xsl:variable name="RBRACE"><xsl:text>}</xsl:text></xsl:variable>
  <xsl:variable name="LBRACKET"><xsl:text>[</xsl:text></xsl:variable>
  <xsl:variable name="RBRACKET"><xsl:text>]</xsl:text></xsl:variable>
  <xsl:variable name="DOLLAR"><xsl:text>$</xsl:text></xsl:variable>
  <xsl:variable name="MINUS"><xsl:text>-</xsl:text></xsl:variable>
  <xsl:variable name="PLUS"><xsl:text>+</xsl:text></xsl:variable>
  <xsl:variable name="EQUAL"><xsl:text>=</xsl:text></xsl:variable>
  <xsl:variable name="COLON"><xsl:text>:</xsl:text></xsl:variable>
  <xsl:variable name="DOUBLE_COLON"><xsl:text>::</xsl:text></xsl:variable>
  <xsl:variable name="SEMICOLON"><xsl:text>;</xsl:text></xsl:variable>
  <xsl:variable name="AT"><xsl:text>@</xsl:text></xsl:variable>
  <xsl:variable name="STAR"><xsl:text>*</xsl:text></xsl:variable>
  <xsl:variable name="QUESTIONMARK"><xsl:text>?</xsl:text></xsl:variable>
  <xsl:variable name="ASSIGN">
    <xsl:value-of select="$COLON"/>
    <xsl:value-of select="$EQUAL"/>
  </xsl:variable>
  <xsl:variable name="SEPARATOR">
    <xsl:value-of select="$SEMICOLON"/>
  </xsl:variable>
  <xsl:variable name="COMMENT_BEGIN">
    <xsl:text>&lt;!--</xsl:text>
  </xsl:variable>
  <xsl:variable name="COMMENT_END">
    <xsl:text>--&gt;</xsl:text>
  </xsl:variable>
  <xsl:variable name="PI_BEGIN">
    <xsl:text>&lt;?</xsl:text>
  </xsl:variable>
  <xsl:variable name="PI_END">
    <xsl:text>?&gt;</xsl:text>
  </xsl:variable>
  <xsl:variable name="CDATA_BEGIN">
    <xsl:text>&lt;![CDATA[</xsl:text>
  </xsl:variable>
  <xsl:variable name="CDATA_END">
    <xsl:text>]&gt;</xsl:text>
  </xsl:variable>
  <xsl:variable name="PRAGMA_BEGIN">
    <xsl:text>(# </xsl:text>
  </xsl:variable>
  <xsl:variable name="PRAGMA_END">
    <xsl:text> #)</xsl:text>
  </xsl:variable>

  <xsl:template name="delimitedList">
    <xsl:param name="delimiter" />
    <xsl:param name="leftEncloser"/>
    <xsl:param name="rightEncloser" />
    <xsl:param name="selector"></xsl:param>

    <xsl:value-of select="$leftEncloser"/>
    <xsl:for-each select="*">
      <xsl:apply-templates select="."/>
      <xsl:if test="not (position()=last())">         
        <xsl:value-of select="$delimiter"/>
      </xsl:if>        
    </xsl:for-each>
    <xsl:value-of select="$rightEncloser"/>
  </xsl:template>    

  <xsl:template name="paranthesizedList">
    <xsl:param name="delimiter" select="$COMMA_SPACE"/>
    <xsl:call-template name="delimitedList">
      <xsl:with-param name="delimiter" select="$delimiter" />
      <xsl:with-param name="leftEncloser" select="$LPAREN"/>
      <xsl:with-param name="rightEncloser" select="$RPAREN"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="commaSeparatedList">
    <xsl:call-template name="delimitedList">
      <xsl:with-param name="delimiter">
        <xsl:value-of select="$COMMA_SPACE"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="quote">
    <xsl:param name="item"/>
    <xsl:value-of select="$DOUBLEQUOTE"/>
    <xsl:value-of select="$item"/>
    <xsl:value-of select="$DOUBLEQUOTE"/>
  </xsl:template>

  <xsl:template match="xqx:*[@xsi:type='xqx:exprWrapper']">
    <xsl:apply-templates select="xqx:expr"/>
  </xsl:template>

  <xsl:template match="xqx:QName">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:NCName">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:rootExpr']">
    <xsl:value-of select="$SLASH"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:contextItemExpr']">
    <xsl:value-of select="$DOT"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:stringConstantExpr']">
    <xsl:call-template name="quote">
      <xsl:with-param name="item" select="xqx:value/text()"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:integerConstantExpr'
      or @xsi:type='xqx:decimalConstantExpr' 
      or xqx:type='xsd:doubleConstantExpr']">
    <xsl:value-of select="xqx:value/text()"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:varRef']">
    <xsl:value-of select="$DOLLAR"/>
    <xsl:value-of select="xqx:name/text()"/>
  </xsl:template>

  <xsl:template match="xqx:pragma">
    <xsl:value-of select="$PRAGMA_BEGIN"/>
    <xsl:value-of select="xqx:pragmaName"/>
    <xsl:value-of select="$SPACE"/>
    <xsl:value-of select="xqx:pragmaContents"/>
    <xsl:value-of select="$PRAGMA_END"/>
  </xsl:template>
  <xsl:template match="xqx:expr[@xsi:type='xqx:extensionExpr']">
    <xsl:apply-templates select="xqx:pragma"/>
    <xsl:value-of select="$LBRACE"/>
    <xsl:apply-templates select="xqx:enclosedExpr"/>
    <xsl:value-of select="$RBRACE"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:functionCallExpr']">
    <xsl:value-of select="xqx:functionName/text()"/>
    <xsl:for-each select="xqx:arguments">
      <xsl:call-template name="paranthesizedList"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:constructorFunctionExpr']">
    <xsl:value-of select="xqx:typeName"/>
    <xsl:for-each select="xqx:arg">
      <xsl:call-template name="paranthesizedList"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="xqx:opType">
    <xsl:choose>
      <xsl:when test="./text() = 'unaryMinus'">
        <xsl:value-of select="$MINUS"/>
      </xsl:when>
      <xsl:when test="./text() = 'unaryPlus'">
        <xsl:value-of select="$PLUS"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="./text()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="operatorArgs">
    <xsl:param name="opType"/>
    <xsl:for-each select="xqx:arguments">
      <xsl:call-template name="paranthesizedList">
        <xsl:with-param name="delimiter">
          <xsl:value-of select="$SPACE"/>
          <xsl:value-of select="$opType"/>
          <xsl:value-of select="$SPACE"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:operatorExpr']">
    <xsl:choose>
      <xsl:when test="xqx:infixOp">
        <xsl:call-template name="operatorArgs">
          <xsl:with-param name="opType" select="xqx:opType"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="xqx:opType"/>
        <xsl:for-each select="xqx:arguments">
          <xsl:call-template name="paranthesizedList"/>
        </xsl:for-each>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:sequenceExpr']">
    <xsl:for-each select="*">
      <xsl:call-template name="paranthesizedList">
        <xsl:with-param name="delimiter" select="$COMMA_NEWLINE"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:rangeSequenceExpr']">
    <xsl:value-of select="$LPAREN"/>
    <xsl:apply-templates select="xqx:startExpr"/>
    <xsl:value-of select="$TO"/>
    <xsl:apply-templates select="xqx:endExpr"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:forClause">
    <xsl:text> for </xsl:text>
    <xsl:call-template name="commaSeparatedList"/>
    <xsl:value-of select="$NEWLINE"/>
  </xsl:template>

  <xsl:template match="xqx:forClauseItem">
    <xsl:apply-templates select="xqx:typedVariableBinding"/>
    <xsl:apply-templates select="xqx:positionalVariableBinding"/>
    <xsl:text> in </xsl:text>
    <xsl:apply-templates select="xqx:forExpr"/>
  </xsl:template>

  <xsl:template match="xqx:letClauseItem">
    <xsl:apply-templates select="xqx:typedVariableBinding"/>
    <xsl:value-of select="$ASSIGN"/>
    <xsl:apply-templates select="xqx:letExpr"/>
  </xsl:template>

  <xsl:template match="xqx:letClause">
    <xsl:text> let </xsl:text>
    <xsl:call-template name="commaSeparatedList"/>
    <xsl:value-of select="$NEWLINE"/>
  </xsl:template>

  <xsl:template match="xqx:returnClause">
    <xsl:text> return </xsl:text>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$NEWLINE"/>
  </xsl:template>

  <xsl:template match="xqx:whereClause">
    <xsl:text> where </xsl:text>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$NEWLINE"/>
  </xsl:template>

  <xsl:template match="xqx:collation">
    <xsl:text> collation </xsl:text>
    <xsl:value-of select="$DOUBLEQUOTE"/>
    <xsl:value-of select="."/>
    <xsl:value-of select="$DOUBLEQUOTE"/>
  </xsl:template>
  <xsl:template match="xqx:emptyOrderingMode">
    <xsl:value-of select="$SPACE"/>
    <xsl:value-of select="."/>
  </xsl:template>
  <xsl:template match="xqx:orderingKind">
    <xsl:value-of select="$SPACE"/>
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:orderModifier">
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <xsl:template match="xqx:orderBySpec">
    <xsl:apply-templates select="xqx:orderByExpr"/>
    <xsl:value-of select="$SPACE"/>
    <xsl:apply-templates select="xqx:orderModifier"/>
  </xsl:template>

  <xsl:template match="xqx:orderByClause">
    <xsl:if test="xqx:stable">
      <xsl:text> stable</xsl:text>
    </xsl:if>
    <xsl:text> order by </xsl:text>
    <xsl:apply-templates select="xqx:orderBySpec[1]"/>
    <xsl:for-each select="xqx:orderBySpec[position() > 1]">
      <xsl:value-of select="$COMMA_SPACE"/>
      <xsl:apply-templates select="."/>
    </xsl:for-each>
    <xsl:value-of select="$NEWLINE"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:flworExpr']">
    <xsl:value-of select="$NEWLINE"/>
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:ifThenElseExpr']">
    <xsl:value-of select="$LPAREN"/>
    <xsl:text> if </xsl:text>
    <xsl:apply-templates select="xqx:ifClause"/>
    <xsl:text> then </xsl:text>
    <xsl:apply-templates select="xqx:thenClause"/>
    <xsl:text> else </xsl:text>
    <xsl:apply-templates select="xqx:elseClause"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:positionalVariableBinding">
    <xsl:text> at </xsl:text>
    <xsl:value-of select="$DOLLAR"/>
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:variableBinding">
    <xsl:value-of select="$DOLLAR"/>
    <xsl:value-of select="."/>
  </xsl:template>
  <xsl:template match="xqx:typedVariableBinding" name="typedVariableBinding">
    <xsl:value-of select="$DOLLAR"/>
    <xsl:value-of select="xqx:varName"/>
    <xsl:apply-templates select="xqx:typeDeclaration"/>
  </xsl:template>

  <xsl:template match="xqx:quantifiedExprInClause">
    <xsl:apply-templates select="xqx:typedVariableBinding"/>
    <xsl:text> in </xsl:text>
    <xsl:apply-templates select="xqx:sourceExpr"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:quantifiedExpr']">
    <xsl:value-of select="$LPAREN"/>
    <xsl:value-of select="xqx:quantifier"/>
    <xsl:value-of select="$SPACE"/>
    <xsl:apply-templates select="xqx:quantifiedExprInClause[1]"/>
    <xsl:for-each select="xqx:quantifiedExprInClause[position() > 1]">
      <xsl:value-of select="$COMMA_SPACE"/>
      <xsl:apply-templates select="."/>
    </xsl:for-each>
    <xsl:text> satisfies </xsl:text>
    <xsl:apply-templates select="xqx:predicateExpr"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:instanceOfExpr']">
    <xsl:value-of select="$LPAREN"/>
    <xsl:apply-templates select="xqx:argExpr"/>
    <xsl:text> instance of </xsl:text>
    <xsl:apply-templates select="xqx:sequenceType"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:castExpr']">
    <xsl:value-of select="$LPAREN"/>
    <xsl:apply-templates select="xqx:argExpr"/>
    <xsl:text> cast as </xsl:text>
    <xsl:apply-templates select="xqx:singleType"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:castableExpr']">
    <xsl:value-of select="$LPAREN"/>
    <xsl:apply-templates select="xqx:argExpr"/>
    <xsl:text> castable as </xsl:text>
    <xsl:apply-templates select="xqx:singleType"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:treatExpr']">
    <xsl:value-of select="$LPAREN"/>
    <xsl:apply-templates select="xqx:argExpr"/>
    <xsl:text> treat as </xsl:text>
    <xsl:apply-templates select="xqx:sequenceType"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>
    <xsl:template match="xqx:typeswitchExprCaseClause">
    <xsl:text> case </xsl:text>
    <xsl:apply-templates select="xqx:variableBinding"/>
    <xsl:apply-templates select="xqx:typeDeclaration"/>
    <xsl:text> return </xsl:text>
    <xsl:apply-templates select="xqx:resultExpr"/>
  </xsl:template>

  <xsl:template match="xqx:typeswitchExprDefaultClause">
    <xsl:text> default </xsl:text>
    <xsl:apply-templates select="xqx:variableBinding"/>
    <xsl:text> return </xsl:text>
    <xsl:apply-templates select="xqx:resultExpr"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:typeswitchExpr']">
    <xsl:value-of select="$LPAREN"/>
    <xsl:text>typeswitch</xsl:text>
    <xsl:value-of select="$LPAREN"/>
    <xsl:apply-templates select="xqx:argExpr"/>
    <xsl:value-of select="$RPAREN"/>
    <xsl:apply-templates select="xqx:typeswitchExprCaseClause"/>
    <xsl:apply-templates select="xqx:typeswitchExprDefaultClause"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:validateExpr']">
    <xsl:value-of select="$LPAREN"/>
    <xsl:text> validate </xsl:text>
    <xsl:value-of select="xqx:validationMode"/>
    <xsl:value-of select="$LBRACE"/>
    <xsl:apply-templates select="xqx:argExpr"/>
    <xsl:value-of select="$RBRACE"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:xpathAxis">
    <xsl:value-of select="."/>
    <xsl:value-of select="$DOUBLE_COLON"/>
  </xsl:template>

  <xsl:template match="xqx:predicates">
    <xsl:for-each select="*">
      <xsl:value-of select="$LBRACKET"/>
      <xsl:apply-templates select="."/>
      <xsl:value-of select="$RBRACKET"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="xqx:star">
    <xsl:value-of select="$STAR"/>
  </xsl:template>

  <xsl:template match="xqx:Wildcard">
    <xsl:apply-templates select="*[1]"/>
    <xsl:if test="*[2]">
      <xsl:value-of select="$COLON"/>
      <xsl:apply-templates select="*[2]"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="simpleWildcard" match="xqx:simpleWildcard">
    <xsl:apply-templates select="xqx:star"/>
    <xsl:apply-templates select="xqx:QName"/>
  </xsl:template>

  <xsl:template match="xqx:textTest">
    <xsl:text>text()</xsl:text>
  </xsl:template>
  <xsl:template match="xqx:commentTest">
    <xsl:text>comment()</xsl:text>
  </xsl:template>
  <xsl:template match="xqx:anyKindTest">
    <xsl:text>node()</xsl:text>
  </xsl:template>
  <xsl:template match="xqx:piTest">
    <xsl:text>processing-instruction</xsl:text>
    <xsl:value-of select="$LPAREN"/>
    <xsl:value-of select="*"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>
  <xsl:template match="xqx:documentTest">
    <xsl:text>document-node</xsl:text>
    <xsl:value-of select="$LPAREN"/>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>
  <xsl:template match="xqx:nameTest">
    <xsl:value-of select="."/>
  </xsl:template>
  <xsl:template match="xqx:attributeTest">
    <xsl:text>attribute</xsl:text>
    <xsl:value-of select="$LPAREN"/>
    <xsl:for-each select="xqx:attributeName">
      <xsl:call-template name="simpleWildcard"/>
    </xsl:for-each>
    <xsl:if test="xqx:typeName">
      <xsl:value-of select="$COMMA"/>
      <xsl:value-of select="xqx:typeName"/>
    </xsl:if>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:elementTest">
    <xsl:text>element</xsl:text>
    <xsl:value-of select="$LPAREN"/>
    <xsl:for-each select="xqx:elementName">
      <xsl:call-template name="simpleWildcard"/>
    </xsl:for-each>
    <xsl:if test="xqx:typeName">
      <xsl:value-of select="$COMMA"/>
      <xsl:value-of select="xqx:typeName"/>
    </xsl:if>
    <xsl:if test="xqx:nillable">
      <xsl:value-of select="$QUESTIONMARK"/>
    </xsl:if>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:schemaElementTest">
    <xsl:text>schema-element</xsl:text>
    <xsl:value-of select="$LPAREN"/>
    <xsl:value-of select="."/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>
  <xsl:template match="xqx:schemaAttributeTest">
    <xsl:text>schema-attribute</xsl:text>
    <xsl:value-of select="$LPAREN"/>
    <xsl:value-of select="."/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:stepExpr">
    <xsl:value-of select="$SLASH"/>
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:pathExpr']">
    <xsl:apply-templates select="xqx:argExpr"/>
    <xsl:apply-templates select="xqx:predicates"/>
    <xsl:apply-templates select="xqx:stepExpr"/>
  </xsl:template>

  <xsl:template match="xqx:attributeConstructor">
    <xsl:value-of select="$SPACE"/>
    <xsl:value-of select="xqx:attributeName"/>
    <xsl:value-of select="$EQUAL"/>
    <xsl:value-of select="$DOUBLEQUOTE"/>
    <xsl:choose>
      <xsl:when test="xqx:attributeValue">
        <xsl:value-of select="xqx:attributeValue"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$LBRACE"/>
        <xsl:apply-templates select="xqx:attributeValueExpr"/>
        <xsl:value-of select="$RBRACE"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="$DOUBLEQUOTE"/>
  </xsl:template>

  <xsl:template match="xqx:attributeList">
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <xsl:template match="xqx:elementContent">
    <xsl:for-each select="*">
      <xsl:value-of select="$LBRACE"/>
      <xsl:apply-templates select="*"/>
      <xsl:value-of select="$RBRACE"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:elementConstructor']">
    <xsl:value-of select="$LESSTHAN"/>
    <xsl:value-of select="xqx:tagName"/>
    <xsl:apply-templates select="xqx:attributeList"/>
    <xsl:value-of select="$GREATERTHAN"/>
    <xsl:apply-templates select="xqx:elementContent"/>
    <xsl:value-of select="$LESSTHAN"/>
    <xsl:value-of select="$SLASH"/>
    <xsl:value-of select="xqx:tagName"/>
    <xsl:value-of select="$GREATERTHAN"/>
  </xsl:template>

  <xsl:template match="xqx:tagNameExpr">
    <xsl:value-of select="$LBRACE"/>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$RBRACE"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:computedElementConstructor']">
    <xsl:text> element </xsl:text>
    <xsl:apply-templates select="xqx:tagName"/>
    <xsl:apply-templates select="xqx:tagNameExpr"/>
    <xsl:value-of select="$LBRACE"/>
    <xsl:apply-templates select="xqx:contentExpr"/>     
    <xsl:value-of select="$RBRACE"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:computedAttributeConstructor']">
    <xsl:text> attribute </xsl:text>
    <xsl:apply-templates select="xqx:tagName"/>
    <xsl:apply-templates select="xqx:tagNameExpr"/>
    <xsl:value-of select="$LBRACE"/>
    <xsl:apply-templates select="xqx:valueExpr"/>     
    <xsl:value-of select="$RBRACE"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:computedDocumentConstructor']">
    <xsl:text> document {</xsl:text>
    <xsl:apply-templates select="*"/>
    <xsl:text> }</xsl:text>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:computedTextConstructor']">
    <xsl:text> text</xsl:text>
    <xsl:value-of select="$LBRACE"/>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$RBRACE"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:computedCommentConstructor']">
    <xsl:text> comment</xsl:text>
    <xsl:value-of select="$LBRACE"/>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$RBRACE"/>
  </xsl:template>

  <xsl:template match="xqx:piTargetExpr">
    <xsl:value-of select="$LBRACE"/>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$RBRACE"/>
  </xsl:template>
  <xsl:template match="xqx:piValueExpr">
    <xsl:value-of select="$LBRACE"/>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$RBRACE"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:computedPIConstructor']">
    <xsl:text> processing-instruction </xsl:text>
    <xsl:value-of select="xqx:piTarget"/>
    <xsl:apply-templates select="xqx:piTargetExpr"/>
    <xsl:apply-templates select="xqx:piValueExpr"/>
  </xsl:template>

  <xsl:template match="xqx:expr[@xsi:type='xqx:unorderedExpr']">
    <xsl:text> unordered( </xsl:text>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>
  <xsl:template match="xqx:expr[@xsi:type='xqx:orderedExpr']">
    <xsl:text> ordered( </xsl:text>
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$RPAREN"/>
  </xsl:template>

  <xsl:template match="xqx:versionDecl">
    <xsl:text> xquery version </xsl:text>
    <xsl:call-template name="quote">
      <xsl:with-param name="item" select="xqx:version"/>
    </xsl:call-template>
    <xsl:value-of select="$SEPARATOR"/>
    <xsl:value-of select="$NEWLINE"/>
  </xsl:template>

  <xsl:template match="xqx:namespaceDecl">
    <xsl:text> declare namespace </xsl:text>
    <xsl:value-of select="xqx:prefix"/>
    <xsl:value-of select="$EQUAL"/>
    <xsl:call-template name="quote">
      <xsl:with-param name="item" select="xqx:uri"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="xqx:defaultNamespaceDecl">
    <xsl:text> declare default </xsl:text>
    <xsl:value-of select="xqx:defaultNamespaceCategory"/>
    <xsl:text> namespace </xsl:text>
    <xsl:call-template name="quote">
      <xsl:with-param name="item" select="xqx:uri"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="xqx:defaultCollationDecl">
    <xsl:text> default collation </xsl:text>
    <xsl:call-template name="quote">
      <xsl:with-param name="item" select="."/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="xqx:baseUriDecl">
    <xsl:text> declare base-uri </xsl:text>
    <xsl:call-template name="quote">
      <xsl:with-param name="item" select="."/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="xqx:boundarySpaceDecl">
    <xsl:text> declare boundary-space </xsl:text>
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:constructionDecl">
    <xsl:text> declare construction </xsl:text>
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:orderingModeDecl">
    <xsl:text> declare ordering </xsl:text>
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:emptyOrderingDecl">
    <xsl:text> declare default ordering </xsl:text>
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:copyNamespacesDecl">
    <xsl:text> declare copy-namespaces </xsl:text>
    <xsl:value-of select="xqx:preserveMode"/>
    <xsl:value-of select="$COMMA"/>
    <xsl:value-of select="xqx:inheritMode"/>
  </xsl:template>

  <xsl:template match="xqx:optionDecl">
    <xsl:text> declare option </xsl:text>
    <xsl:value-of select="xqx:optionName"/>
    <xsl:value-of select="$SPACE"/>
    <xsl:value-of select="xqx:optionContents"/>
  </xsl:template>

  <xsl:template match="xqx:voidSequenceType">
    <xsl:text>void()</xsl:text>
  </xsl:template>

  <xsl:template match="xqx:occurrenceIndicator">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:anyItemType">
    <xsl:text>item()</xsl:text>
  </xsl:template>

  <xsl:template match="xqx:sequenceType">
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <xsl:template match="xqx:atomicType">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="xqx:singleType">
    <xsl:apply-templates select="xqx:atomicType"/>
    <xsl:if test="xqx:optional">
      <xsl:text>?</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="xqx:typeDeclaration">
    <xsl:text> as </xsl:text>
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <xsl:template match="xqx:varDecl">
    <xsl:text> declare variable </xsl:text>
    <xsl:value-of select="$DOLLAR"/>
    <xsl:value-of select="xqx:varName"/>
    <xsl:value-of select="$SPACE"/>
    <xsl:apply-templates select="xqx:typeDeclaration"/>
    <xsl:if test="xqx:external">
      <xsl:text> external </xsl:text>
    </xsl:if>
    <xsl:if test="xqx:varValue">
      <xsl:value-of select="$ASSIGN"/>
      <xsl:apply-templates select="xqx:varValue"/>
    </xsl:if>
  </xsl:template>

  <xsl:template match="xqx:schemaImport">
    <xsl:text> import schema </xsl:text>
    <xsl:if test="xqx:defaultElementNamespace">
      <xsl:text> default element namespace </xsl:text>
    </xsl:if>
    <xsl:if test="xqx:namespacePrefix">
      <xsl:text> namespace </xsl:text>
      <xsl:value-of select="xqx:namespacePrefix"/>
      <xsl:value-of select="$EQUAL"/>
    </xsl:if>
    <xsl:call-template name="quote">
      <xsl:with-param name="item" select="xqx:targetNamespace"/>
    </xsl:call-template>
    <xsl:if test="xqx:targetLocation">
      <xsl:text> at </xsl:text>
      <xsl:call-template name="quote">
        <xsl:with-param name="item" select="xqx:targetLocation"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template match="xqx:moduleImport">
    <xsl:text> import module </xsl:text>
    <xsl:if test="xqx:namespacePrefix">
      <xsl:text> namespace </xsl:text>
      <xsl:value-of select="xqx:namespacePrefix"/>
      <xsl:value-of select="$EQUAL"/>
    </xsl:if>
    <xsl:call-template name="quote">
      <xsl:with-param name="item" select="xqx:targetNamespace"/>
    </xsl:call-template>
    <xsl:if test="xqx:targetLocation">
      <xsl:text> at </xsl:text>
      <xsl:call-template name="quote">
        <xsl:with-param name="item" select="xqx:targetLocation"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template match="xqx:param">
    <xsl:value-of select="$DOLLAR"/>
    <xsl:value-of select="xqx:varName"/>
    <xsl:apply-templates select="xqx:typeDeclaration"/>
  </xsl:template>

  <xsl:template match="xqx:paramList">
    <xsl:call-template name="paranthesizedList"/>
  </xsl:template>

  <xsl:template match="xqx:functionBody">
    <xsl:value-of select="$NEWLINE"/>
    <xsl:value-of select="$LBRACE"/>
    <xsl:value-of select="$NEWLINE"/>
    <xsl:apply-templates select="xqx:expr"/>
    <xsl:value-of select="$NEWLINE"/>
    <xsl:value-of select="$RBRACE"/>
  </xsl:template>

  <xsl:template match="xqx:functionDecl">
    <xsl:text> declare function </xsl:text>
    <xsl:value-of select="xqx:functionName"/>
    <xsl:apply-templates select="xqx:paramList"/>
    <xsl:apply-templates select="xqx:typeDeclaration"/>
    <xsl:apply-templates select="xqx:functionBody"/>
    <xsl:if test="xqx:externalDefinition">
      <xsl:text> external </xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="xqx:queryBody">
    <xsl:apply-templates select="*"/>
    <xsl:value-of select="$NEWLINE"/>
  </xsl:template>

  <xsl:template match="xqx:moduleDecl">
    <xsl:text> module </xsl:text>
    <xsl:value-of select="xqx:prefix"/>
    <xsl:value-of select="$EQUAL"/>
    <xsl:call-template name="quote">
      <xsl:with-param name="item" select="xqx:uri" />
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="xqx:prolog">
    <xsl:for-each select="*">
      <xsl:apply-templates select="."/>
      <xsl:value-of select="$SEPARATOR"/>
      <xsl:value-of select="$NEWLINE"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="xqx:libraryModule">
    <xsl:apply-templates select="xqx:moduleDecl"/>
    <xsl:apply-templates select="xqx:prolog"/>
  </xsl:template>

  <xsl:template match="xqx:mainModule">
    <xsl:apply-templates select="xqx:prolog"/>
    <xsl:apply-templates select="xqx:queryBody"/>
  </xsl:template>

  <xsl:template match="xqx:module">
    <xsl:apply-templates select="*"/>
  </xsl:template>

</xsl:stylesheet>